----------------------------------------------
-- Create the addon object
--
MAcon = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

----------------------------------------------
-- Set up database
--
MAcon:RegisterDB("MAconDB")
MAcon:RegisterDefaults("profile", {
	icon = "7",
	friendlies = false,
})

----------------------------------------------
-- Set up chat commands
--
MAcon:RegisterChatCommand({"/macon", "/ma"}, {
	type = "group",
	name = "MAcon",
	args = {
		icon = {
			name = "Icon",
			desc = "Set which icon to use for MA targetting",
			type = "text",
			get = function() return MAcon.db.profile.icon end,
			set = function(icon) MAcon.db.profile.icon = icon end,
			validate = {
				["0"] = "0-none",
				["1"] = "1-star",
				["2"] = "2-circle",
				["3"] = "3-diamond",
				["4"] = "4-triangle",
				["5"] = "5-moon",
				["6"] = "6-square",
				["7"] = "7-cross",
				["8"] = "8-skull",
			},
		},
		friendlies = {
			name = "Friendlies",
			desc = "Also mark neutral/friendly targets",
			type = "toggle",
			get = function() return MAcon.db.profile.friendlies end,
			set = function(v) MAcon.db.profile.friendlies = v end,
		},
	},
}, "MACON")

----------------------------------------------
-- AceAddon Functions
--
function MAcon:OnEnable()
	self:RegisterEvent("PLAYER_TARGET_CHANGED")

	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "CheckInGroup")
	self:RegisterEvent("RAID_ROSTER_UPDATE", 	"CheckInGroup")
	self:RegisterEvent("PARTY_LEADER_CHANGED", 	"CheckInGroup")

	self:CheckInGroup()
	self:PLAYER_TARGET_CHANGED()
end

function MAcon:PLAYER_TARGET_CHANGED()
	if(UnitExists("target") and not (UnitIsDead("target") or UnitIsUnit("player", "target") or UnitIsPlayer("target"))) then
		if(self.db.profile.friendlies or UnitIsEnemy("target", "player")) then
			self.placenew = true
		else
			self.placenew = false
		end
	else
		self.placenew = false
	end
end

function MAcon:updateMAIcon()
    if(self.placenew) then
        SetRaidTarget("target", self.db.profile.icon)
        self.placed = "target"
	else
		if(self.placed == "target") then
			self:ScheduleRepeatingEvent("MAcon_ClearIcon", self.ClearIcon, 0.5, self)
		end
    end
end

function MAcon:ClearIcon()
	if(self.placed == "target") then
		SetRaidTarget("player", self.db.profile.icon)
		self.placed = "player"
	elseif(self.placed == "player") then
		SetRaidTarget("player", 0)
		self.placed = "none"
		self:CancelScheduledEvent("MAcon_ClearIcon")
	end
end

-- Checks if player is allowed to set raid target icons
function MAcon:CheckInGroup()
	if(self:AllowedIcon()) then
		if(not self:IsEventScheduled("MAcon_Update")) then
			self:ScheduleRepeatingEvent("MAcon_Update", self.updateMAIcon, 2, self)
		end
	else
		self:CancelAllScheduledEvents()
	end
end

function MAcon:AllowedIcon()
	if(GetNumRaidMembers() > 0 and IsRaidLeader() or IsRaidOfficer()) then
		return true
	elseif(GetNumPartyMembers() > 0 and GetNumRaidMembers() == 0 and IsPartyLeader()) then
		return true
	else
		return false
	end
end
